#ifndef _COMMON_H
#define _COMMON_H

#define REGION_NAME_ALL		"all"
#define REGION_NAME_FPGA	"fpga"
#define REGION_NAME_IPL		"ipl"
#define REGION_NAME_BOOTLOADER	"bootloader"
#define REGION_NAME_KERNEL	"kernel"
#define REGION_NAME_USERLAND	"userland"
#define REGION_NAME_RECOVERY	"recovery"
#define REGION_NAME_LICENSE	"license"
#define REGION_NAME_IMAGE	"image"
#define REGION_NAME_CONFIG	"config"
#define REGION_NAME_FREE1	"free1"
#define REGION_NAME_FREE2	"free2"
#define REGION_NAME_ALL_WITHOUT_CONFIG	"all_without_config"

#define KiB            (1024)
#define MiB            (1024 * 1024)

enum {
	REGION_ID_ALL = 0,
	REGION_ID_FPGA,
	REGION_ID_IPL,
	REGION_ID_BOOTLOADER,
	REGION_ID_KERNEL,
	REGION_ID_USERLAND,
	REGION_ID_RECOVERY,
	REGION_ID_LICENSE,
	REGION_ID_IMAGE,
	REGION_ID_CONFIG,
	REGION_ID_FREE1,
	REGION_ID_FREE2,
	REGION_ID_ALL_WITHOUT_CONFIG,
	REGION_ID_MAX_NR
};

enum {
	SZ010 = 0,
	SZ030,
	SZ130,
	SZ310,
	SZ410,
	A9,
	A210,
	A220,
	A230,
	A240,
	A300,
	A420,
	A420_3_14,
	A440,
	A440_3_14,
	A441,
	A441_3_14,
	A440_node_eye,
	A460,
	A461,
	A410,
	A410_3_14,
	A411,
	A411_3_14,
	A410_iotg,
	A410_iotg_node_eye,
	A500,
	A500FX
};

#define BOARD_ID_MAX   (A500FX)

static char *supported_board_names[] =  {"SZ010",
					 "SZ030",
					 "SZ130",
					 "SZ310",
					 "SZ410",
					 "A9",
					 "A210",
					 "A220",
					 "A230",
					 "A240",
					 "A300",
					 "A420",
					 "A420-3.14",
					 "A440",
					 "A440-3.14",
					 "A441",
					 "A441-3.14",
					 "A440-node-eye",
					 "A460",
					 "A461",
					 "A410",
					 "A410-3.14",
					 "A411",
					 "A411-3.14",
					 "A410_iotg",
					 "A410_iotg-node-eye",
					 "A500",
					 "A500FX",
					 NULL};

struct region_pair {
	int   id;
	char *name;
} region_pairs[] = {
	{ REGION_ID_ALL,                REGION_NAME_ALL },
	{ REGION_ID_FPGA,               REGION_NAME_FPGA },
	{ REGION_ID_IPL,                REGION_NAME_IPL },
	{ REGION_ID_BOOTLOADER,         REGION_NAME_BOOTLOADER },
	{ REGION_ID_KERNEL,             REGION_NAME_KERNEL},
	{ REGION_ID_USERLAND,           REGION_NAME_USERLAND},
	{ REGION_ID_RECOVERY,           REGION_NAME_RECOVERY },
	{ REGION_ID_LICENSE,            REGION_NAME_LICENSE },
	{ REGION_ID_IMAGE,              REGION_NAME_IMAGE },
	{ REGION_ID_CONFIG,             REGION_NAME_CONFIG },
	{ REGION_ID_FREE1,              REGION_NAME_FREE1 },
	{ REGION_ID_FREE2,              REGION_NAME_FREE2 },
	{ REGION_ID_ALL_WITHOUT_CONFIG, REGION_NAME_ALL_WITHOUT_CONFIG },
	{ REGION_ID_MAX_NR,     NULL },
};

struct region_addr {
	unsigned long start;
	unsigned long size;
};

struct region {
	char *name;
	struct region_addr addr;
} flash_map[BOARD_ID_MAX+1][REGION_ID_MAX_NR] = {
	{ /* SZ010 */
		{REGION_NAME_ALL,        {0x00000000, (4*MiB)}},
		{REGION_NAME_FPGA,       {0x00000000, (8*64*KiB)}},
		{REGION_NAME_BOOTLOADER, {0x00080000, (2*64*KiB)}},
		{REGION_NAME_IMAGE,      {0x000A0000, (53*64*KiB)}},
		{REGION_NAME_CONFIG,     {0x003F0000, (8*8*KiB)}},
		{REGION_NAME_ALL_WITHOUT_CONFIG, {0x00000000, (4*MiB - 8*8*KiB)}},
		{NULL, {0, 0}},
	},
	{ /* SZ030 */
		{REGION_NAME_ALL,        {0x00000000, (8*MiB)}},
		{REGION_NAME_FREE1,      {0x00000000, (8*8*KiB)}},
		{REGION_NAME_FREE2,      {0x00010000, (7*64*KiB)}},
		{REGION_NAME_FPGA,       {0x00080000, (8*64*KiB)}},
		{REGION_NAME_BOOTLOADER, {0x00100000, (2*64*KiB)}},
		{REGION_NAME_IMAGE,      {0x00120000, (109*64*KiB)}},
		{REGION_NAME_CONFIG,     {0x007F0000, (8*8*KiB)}},
		{REGION_NAME_ALL_WITHOUT_CONFIG, {0x00000000, (8*MiB - 8*8*KiB)}},
		{NULL, {0, 0}},
	},
	{ /* SZ130 */
		{REGION_NAME_ALL,        {0x00000000, (8*MiB)}},
		{REGION_NAME_FPGA,       {0x00000000, (16*64*KiB)}},
		{REGION_NAME_BOOTLOADER, {0x00100000, (2*64*KiB)}},
		{REGION_NAME_IMAGE,      {0x00120000, (109*64*KiB)}},
		{REGION_NAME_CONFIG,     {0x007F0000, (1*64*KiB)}},
		{REGION_NAME_ALL_WITHOUT_CONFIG, {0x00000000, (8*MiB - 64*KiB)}},
		{NULL, {0, 0}},
	},
	{ /* SZ310 */
		{REGION_NAME_ALL,        {0x00000000, (8*MiB)}},
		{REGION_NAME_FREE1,      {0x00000000, (8*8*KiB)}},
		{REGION_NAME_FREE2,      {0x00010000, (7*64*KiB)}},
		{REGION_NAME_FPGA,       {0x00080000, (8*64*KiB)}},
		{REGION_NAME_BOOTLOADER, {0x00100000, (2*64*KiB)}},
		{REGION_NAME_IMAGE,      {0x00120000, (109*64*KiB)}},
		{REGION_NAME_CONFIG,     {0x007F0000, (8*8*KiB)}},
		{REGION_NAME_ALL_WITHOUT_CONFIG, {0x00000000, (8*MiB - 8*8*KiB)}},
		{NULL, {0, 0}},
	},
	{ /* SZ410 */
		{REGION_NAME_ALL,        {0x00000000, (8*MiB)}},
		{REGION_NAME_FPGA,       {0x00000000, (16*64*KiB)}},
		{REGION_NAME_BOOTLOADER, {0x00100000, (2*64*KiB)}},
		{REGION_NAME_IMAGE,      {0x00120000, (109*64*KiB)}},
		{REGION_NAME_CONFIG,     {0x007F0000, (1*64*KiB)}},
		{REGION_NAME_ALL_WITHOUT_CONFIG, {0x00000000, (8*MiB - 64*KiB)}},
		{NULL, {0, 0}},
	},
	{ /* A9 */
		{REGION_NAME_ALL,        {0x00000000, (8*MiB)}},
		{REGION_NAME_BOOTLOADER, {0x00000000, (64*KiB)}},
		{REGION_NAME_KERNEL,     {0x00010000, (1472*KiB)}},
		{REGION_NAME_USERLAND,   {0x00180000, (6592*KiB)}},
		{REGION_NAME_CONFIG,     {0x007F0000, (1*64*KiB)}},
		{REGION_NAME_ALL_WITHOUT_CONFIG, {0x00000000, (8*MiB - 64*KiB)}},
		{NULL, {0, 0}},
	},
	{ /* A210 */
		{REGION_NAME_ALL,        {0x00000000, (4*MiB)}},
		{REGION_NAME_BOOTLOADER, {0x00000000, (64*KiB)}},
		{REGION_NAME_KERNEL,     {0x00010000, (1472*KiB)}},
		{REGION_NAME_USERLAND,   {0x00180000, (2496*KiB)}},
		{REGION_NAME_CONFIG,     {0x003F0000, (1*64*KiB)}},
		{REGION_NAME_ALL_WITHOUT_CONFIG, {0x00000000, (4*MiB - 64*KiB)}},
		{NULL, {0, 0}},
	},
	{ /* A220 */
		{REGION_NAME_ALL,        {0x00000000, (8*MiB)}},
		{REGION_NAME_BOOTLOADER, {0x00000000, (64*KiB)}},
		{REGION_NAME_KERNEL,     {0x00010000, (1472*KiB)}},
		{REGION_NAME_USERLAND,   {0x00180000, (6592*KiB)}},
		{REGION_NAME_CONFIG,     {0x007F0000, (1*64*KiB)}},
		{REGION_NAME_ALL_WITHOUT_CONFIG, {0x00000000, (8*MiB - 64*KiB)}},
		{NULL, {0, 0}},
	},
	{ /* A230 */
		{REGION_NAME_ALL,        {0x00000000, (8*MiB)}},
		{REGION_NAME_BOOTLOADER, {0x00000000, (64*KiB)}},
		{REGION_NAME_KERNEL,     {0x00010000, (1472*KiB)}},
		{REGION_NAME_USERLAND,   {0x00180000, (6592*KiB)}},
		{REGION_NAME_CONFIG,     {0x007F0000, (1*64*KiB)}},
		{REGION_NAME_ALL_WITHOUT_CONFIG, {0x00000000, (8*MiB - 64*KiB)}},
		{NULL, {0, 0}},
	},
	{ /* A240 */
		{REGION_NAME_ALL,        {0x00000000, (8*MiB)}},
		{REGION_NAME_BOOTLOADER, {0x00000000, (64*KiB)}},
		{REGION_NAME_KERNEL,     {0x00010000, (1472*KiB)}},
		{REGION_NAME_USERLAND,   {0x00180000, (6592*KiB)}},
		{REGION_NAME_CONFIG,     {0x007F0000, (1*64*KiB)}},
		{REGION_NAME_ALL_WITHOUT_CONFIG, {0x00000000, (8*MiB - 64*KiB)}},
		{NULL, {0, 0}},
	},
	{ /* A300 */
		{REGION_NAME_ALL,        {0x00000000, (8*MiB)}},
		{REGION_NAME_IPL,        {0x00000000, (8*KiB)}},
		{REGION_NAME_BOOTLOADER, {0x00002000, (56*KiB)}},
		{REGION_NAME_KERNEL,     {0x00010000, (2048*KiB)}},
		{REGION_NAME_USERLAND,   {0x00210000, (6016*KiB)}},
		{REGION_NAME_CONFIG,     {0x007F0000, (1*64*KiB)}},
		{REGION_NAME_ALL_WITHOUT_CONFIG, {0x00000000, (8*MiB - 64*KiB)}},
		{NULL, {0, 0}},
	},
	{ /* A420 */
		{REGION_NAME_ALL,        {0x00000000, (16*MiB)}},
		{REGION_NAME_BOOTLOADER, {0x00000000, (128*KiB)}},
		{REGION_NAME_KERNEL,     {0x00020000, (2048*KiB)}},
		{REGION_NAME_USERLAND,   {0x00220000, (14080*KiB)}},
		{REGION_NAME_CONFIG,     {0x00FE0000, (128*KiB)}},
		{REGION_NAME_ALL_WITHOUT_CONFIG, {0x00000000, (16*MiB - 128*KiB)}},
		{NULL, {0, 0}},
	},
	{ /* A420-3.14 */
		{REGION_NAME_ALL,        {0x00000000, (16*MiB)}},
		{REGION_NAME_BOOTLOADER, {0x00000000, (128*KiB)}},
		{REGION_NAME_KERNEL,     {0x00020000, (4096*KiB)}},
		{REGION_NAME_USERLAND,   {0x00420000, (11136*KiB)}},
		{REGION_NAME_CONFIG,     {0x00F00000, (1024*KiB)}},
		{REGION_NAME_ALL_WITHOUT_CONFIG, {0x00000000, (16*MiB - 1024*KiB)}},
		{NULL, {0, 0}},
	},
	{ /* A440 */
		{REGION_NAME_ALL,        {0x00000000, (32*MiB)}},
		{REGION_NAME_BOOTLOADER, {0x00000000, (128*KiB)}},
		{REGION_NAME_KERNEL,     {0x00020000, (2048*KiB)}},
		{REGION_NAME_USERLAND,   {0x00220000, (30464*KiB)}},
		{REGION_NAME_CONFIG,     {0x01FE0000, (128*KiB)}},
		{REGION_NAME_ALL_WITHOUT_CONFIG, {0x00000000, (32*MiB - 128*KiB)}},
		{NULL, {0, 0}},
	},
	{ /* A440-3.14 */
		{REGION_NAME_ALL,        {0x00000000, (32*MiB)}},
		{REGION_NAME_BOOTLOADER, {0x00000000, (128*KiB)}},
		{REGION_NAME_KERNEL,     {0x00020000, (4096*KiB)}},
		{REGION_NAME_USERLAND,   {0x00420000, (27520*KiB)}},
		{REGION_NAME_CONFIG,     {0x01F00000, (1024*KiB)}},
		{REGION_NAME_ALL_WITHOUT_CONFIG, {0x00000000, (32*MiB - 1024*KiB)}},
		{NULL, {0, 0}},
	},
	{ /* A441 */
		{REGION_NAME_ALL,        {0x00000000, (32*MiB)}},
		{REGION_NAME_BOOTLOADER, {0x00000000, (256*KiB)}},
		{REGION_NAME_KERNEL,     {0x00040000, (2048*KiB)}},
		{REGION_NAME_USERLAND,   {0x00240000, (30336*KiB)}},
		{REGION_NAME_CONFIG,     {0x01FE0000, (128*KiB)}},
		{REGION_NAME_ALL_WITHOUT_CONFIG, {0x00000000, (32*MiB - 128*KiB)}},
		{NULL, {0, 0}},
	},
	{ /* A441-3.14 */
		{REGION_NAME_ALL,        {0x00000000, (32*MiB)}},
		{REGION_NAME_BOOTLOADER, {0x00000000, (256*KiB)}},
		{REGION_NAME_KERNEL,     {0x00040000, (4096*KiB)}},
		{REGION_NAME_USERLAND,   {0x00440000, (27392*KiB)}},
		{REGION_NAME_CONFIG,     {0x01F00000, (1024*KiB)}},
		{REGION_NAME_ALL_WITHOUT_CONFIG, {0x00000000, (32*MiB - 1024*KiB)}},
		{NULL, {0, 0}},
	},
	{ /* A440-node-eye */
		{REGION_NAME_ALL,        {0x00000000, (32*MiB)}},
		{REGION_NAME_BOOTLOADER, {0x00000000, (128*KiB)}},
		{REGION_NAME_RECOVERY,   {0x00020000, (8192*KiB)}},
		{REGION_NAME_KERNEL,     {0x00820000, (4096*KiB)}},
		{REGION_NAME_USERLAND,   {0x00C20000, (19200*KiB)}},
		{REGION_NAME_LICENSE,    {0x01EE0000, (128*KiB)}},
		{REGION_NAME_CONFIG,     {0x01F00000, (1024*KiB)}},
		{REGION_NAME_ALL_WITHOUT_CONFIG, {0x00000000, (32*MiB - 1024*KiB)}},
		{NULL, {0, 0}},
	},
	{ /* A460 */
		{REGION_NAME_ALL,        {0x00000000, (32*MiB)}},
		{REGION_NAME_BOOTLOADER, {0x00000000, (128*KiB)}},
		{REGION_NAME_KERNEL,     {0x00020000, (2048*KiB)}},
		{REGION_NAME_USERLAND,   {0x00220000, (30464*KiB)}},
		{REGION_NAME_CONFIG,     {0x01FE0000, (128*KiB)}},
		{REGION_NAME_ALL_WITHOUT_CONFIG, {0x00000000, (32*MiB - 128*KiB)}},
		{NULL, {0, 0}},
	},
	{ /* A461 */
		{REGION_NAME_ALL,        {0x00000000, (32*MiB)}},
		{REGION_NAME_BOOTLOADER, {0x00000000, (256*KiB)}},
		{REGION_NAME_KERNEL,     {0x00040000, (2048*KiB)}},
		{REGION_NAME_USERLAND,   {0x00240000, (30336*KiB)}},
		{REGION_NAME_CONFIG,     {0x01FE0000, (128*KiB)}},
		{REGION_NAME_ALL_WITHOUT_CONFIG, {0x00000000, (32*MiB - 128*KiB)}},
		{NULL, {0, 0}},
	},
	{ /* A410 */
		{REGION_NAME_ALL,        {0x00000000, (32*MiB)}},
		{REGION_NAME_BOOTLOADER, {0x00000000, (128*KiB)}},
		{REGION_NAME_KERNEL,     {0x00020000, (2048*KiB)}},
		{REGION_NAME_USERLAND,   {0x00220000, (30464*KiB)}},
		{REGION_NAME_CONFIG,     {0x01FE0000, (128*KiB)}},
		{REGION_NAME_ALL_WITHOUT_CONFIG, {0x00000000, (32*MiB - 128*KiB)}},
		{NULL, {0, 0}},
	},
	{ /* A410-3.14 */
		{REGION_NAME_ALL,        {0x00000000, (32*MiB)}},
		{REGION_NAME_BOOTLOADER, {0x00000000, (128*KiB)}},
		{REGION_NAME_KERNEL,     {0x00020000, (4096*KiB)}},
		{REGION_NAME_USERLAND,   {0x00420000, (27520*KiB)}},
		{REGION_NAME_CONFIG,     {0x01F00000, (1024*KiB)}},
		{REGION_NAME_ALL_WITHOUT_CONFIG, {0x00000000, (32*MiB - 1024*KiB)}},
		{NULL, {0, 0}},
	},
	{ /* A411 */
		{REGION_NAME_ALL,        {0x00000000, (32*MiB)}},
		{REGION_NAME_BOOTLOADER, {0x00000000, (256*KiB)}},
		{REGION_NAME_KERNEL,     {0x00040000, (2048*KiB)}},
		{REGION_NAME_USERLAND,   {0x00240000, (30336*KiB)}},
		{REGION_NAME_CONFIG,     {0x01FE0000, (128*KiB)}},
		{REGION_NAME_ALL_WITHOUT_CONFIG, {0x00000000, (32*MiB - 128*KiB)}},
		{NULL, {0, 0}},
	},
	{ /* A411-3.14 */
		{REGION_NAME_ALL,        {0x00000000, (32*MiB)}},
		{REGION_NAME_BOOTLOADER, {0x00000000, (256*KiB)}},
		{REGION_NAME_KERNEL,     {0x00040000, (4096*KiB)}},
		{REGION_NAME_USERLAND,   {0x00440000, (27392*KiB)}},
		{REGION_NAME_CONFIG,     {0x01F00000, (1024*KiB)}},
		{REGION_NAME_ALL_WITHOUT_CONFIG, {0x00000000, (32*MiB - 1024*KiB)}},
		{NULL, {0, 0}},
	},
	{ /* A410_IoTG */
		{REGION_NAME_ALL,        {0x00000000, (32*MiB)}},
		{REGION_NAME_BOOTLOADER, {0x00000000, (128*KiB)}},
		{REGION_NAME_KERNEL,     {0x00020000, (4096*KiB)}},
		{REGION_NAME_USERLAND,   {0x00420000, (27520*KiB)}},
		{REGION_NAME_CONFIG,     {0x01F00000, (1024*KiB)}},
		{REGION_NAME_ALL_WITHOUT_CONFIG, {0x00000000, (32*MiB - 1024*KiB)}},
		{NULL, {0, 0}},
	},
	{ /* A410_IoTG-node-eye */
		{REGION_NAME_ALL,        {0x00000000, (32*MiB)}},
		{REGION_NAME_BOOTLOADER, {0x00000000, (128*KiB)}},
		{REGION_NAME_RECOVERY,   {0x00020000, (8192*KiB)}},
		{REGION_NAME_KERNEL,     {0x00820000, (4096*KiB)}},
		{REGION_NAME_USERLAND,   {0x00C20000, (19200*KiB)}},
		{REGION_NAME_LICENSE,    {0x01EE0000, (128*KiB)}},
		{REGION_NAME_CONFIG,     {0x01F00000, (1024*KiB)}},
		{REGION_NAME_ALL_WITHOUT_CONFIG, {0x00000000, (32*MiB - 1024*KiB)}},
		{NULL, {0, 0}},
	},
	{ /* A500 */
		{REGION_NAME_ALL,        {0x00000000, (16*MiB)}},
		{REGION_NAME_BOOTLOADER, {0x00000000, (128*KiB)}},
		{REGION_NAME_KERNEL,     {0x00020000, (2048*KiB)}},
		{REGION_NAME_USERLAND,   {0x00220000, (14080*KiB)}},
		{REGION_NAME_CONFIG,     {0x00FE0000, (128*KiB)}},
		{REGION_NAME_ALL_WITHOUT_CONFIG, {0x00000000, (16*MiB - 128*KiB)}},
		{NULL, {0, 0}},
	},
	{ /* A500FX */
		{REGION_NAME_ALL,        {0x00000000, (32*MiB)}},
		{REGION_NAME_BOOTLOADER, {0x00000000, (128*KiB)}},
		{REGION_NAME_KERNEL,     {0x00020000, (2048*KiB)}},
		{REGION_NAME_USERLAND,   {0x00220000, (30464*KiB)}},
		{REGION_NAME_CONFIG,     {0x00FE0000, (128*KiB)}},
		{REGION_NAME_ALL_WITHOUT_CONFIG, {0x00000000, (32*MiB - 128*KiB)}},
		{NULL, {0, 0}},
	}
};

static char *program_name;

#define LOG_ERROR(format, ...) ({					\
			if (!quiet)					\
				fprintf(stderr, "%s[error]: "format,	\
					program_name, ##__VA_ARGS__);	\
		})
#define LOG_INFO(format, ...) ({					\
			if (!quiet)					\
				printf("%s[info]: "format,		\
				       program_name, ##__VA_ARGS__);	\
		})

#ifdef DEBUG
#define LOG_DEBUG(format, ...) ({printf("%s[debug]: "format,		\
					program_name, ##__VA_ARGS__);})
#else
#define LOG_DEBUG(format, ...)
#endif

#endif /* _COMMON_H */
